/*--------------------------------------------------------------------------*/
/** @file device.h
@brief  High-level interaction with the device bearing the manager functions 
which implement the algorithms of modes.

                  MASS project: TURBINA module file
                       header file for device.cpp

The algorithms represent the sequences of actions which implement the one 
accumulation-time cycle of measurements
by means provided by counter.h, motor.h, power.h, tracker.h and light.h.



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef DEVICE_H
#define DEVICE_H

#include <qobject.h>
//#include <qdatetime.h>
#include "exchange.h"
#include "counter.h"
#include "light.h"
#include "motor.h"
#include "tracker.h"
#include "power.h"
#include "scan.hxx"
#include "trbtime.h"

#define TM_PREC			(1.5)	// time precision for count timeout interval
//#define GAP_RA			2.0	// sec vremeni tracking po RA
//#define GAP_DEC			2.0	// sec vremeni tracking po Dec
//#define MAX_CORRECT_ITER	5	// 3 ?

// not scenario:
#define SCENARIO_MODE			0
#define PLAYBACK_MODE			1
#define BACKGROUND_CORR_MODE		2
#define CENTERING_CORR_MODE		3
#define FLUX_ESTIMATION_MODE		4
#define TRACK_CALIBRATION_MODE		5

// Scenario-modes:
#define EXCHANGE_TEST_MODE		6
#define DETECTOR_TEST_MODE		7
#define STATISTIC_TEST_MODE		8
#define DETECTOR_COUNTING_MODE		9
#define DETECTOR_STATISTIC_MODE		10
// work:
#define FOCUS_CONTROL_MODE		11
#define CENTER_CONTROL_MODE		12
#define CENTERING_MODE			13
#define NORMAL_MODE			14
#define GENERALIZED_MODE		15
#define BACKGROUND_MEASUR_MODE		16

// FINISH:
#define QUIT_PROGRAM			17
#define END_OF_PROGRAM			18

#define UNDEFINED_MODE			19

typedef double matrix43[MAX_N_COUNTERS][3];

class PointDouble {
private:
	double _x;
	double _y;
public:
	PointDouble( void ) { _x = 0; _y = 0; }
	~PointDouble() {}
	double x( void ) { return _x; }
	double y( void ) { return _y; }
	void setX( double newX ) { _x = newX; }
	void setY( double newY ) { _y = newY; }
};

void	timeout_action( void );
void	*read_status( void *arg );
void	*read_data( void );
void	terminate_program( void );
void*	count_cycle( void );
void	initThreads( void );

void	sendToScreen( const QString& text );
void	sendToOutFile( const QString& text,
			const char* prefix="> " );
void	sendErrorInfo( const QString& text );


class DevManager : public QObject {
	Q_OBJECT

protected:
	void timerEvent( QTimerEvent* );

private:
	Counter *masterCounter;
	Counter *slaveCounter1;
	Counter *slaveCounter2;
	Counter *slaveCounter3;

	Counter *testCounter;

	scan_t scan[MAX_N_COUNTERS];

	Counter counters[MAX_N_COUNTERS];
	Light light;
	Motor motor;
	Tracker tracker;
	Power power;

	bool	_withAutoGuide;

	int	_polling;

	int	_temperPolling;
	int	_overLightPolling;
	int	_buttonsPolling;
	int	_baseTimeTimerPolling;

	int	_baseTimeTimer;
	int	_temperTimer;
	int	_hvTimer;
	int	_buttonsTimer;

	int	_testOverLight_flag;
	int	_processButtons_flag;
	int	_getTemper_flag;

// Flagi sostoyaniya:
	bool	isOverLight;
	bool	isScenario;
	bool	_isCounting;
	bool	_isPlayBack;
	bool	_isTracking;
	bool	isUserStop;
	bool	isFirst;	// true if first loadConst & init
	bool	isFirstInit;
	bool	isParking;	// true poka parkuyetsia
	bool	isQuiting;

	const char* _name_A;
	const char* _name_B;
	const char* _name_C;
	const char* _name_D;

	int _total_test_err1,_total_test_err2,_total_test_err3,_total_test_err4; 
	double _zShiftNormal,_zShiftGeneral,_zShiftGeneralSecond;

// Files & Dir-s
	int		_sciCountSave;		//	sci-mode: save to file or not
	int		_techCountSave;	//	tech-mode: save to file or not
	int		_logSave;		//	save to file or not
	QString		_logDirName;
	QString		_dataDirName;
	QString		_checkFile;	//for atmos 
	QString		_sumFileName;	// summary
	QString		_dumpFileName;	// summary
	QString		_massInputFileName; // Play Back

// Modes
	int	_dataBlockSize;
	int	_dataBlockSize0;
	int	_normalBlockNumber;
	int	_generalBlockNumber,_generalSecondBlockNumber;
	int	_focusBlockNumber;
	int	_centerBlockNumber;

	double	_gapRA; // for tracker
	double	_gapDec; // for tracker

	double	_normalExposition;
	double	_generalExposition;
	double	_focusExposition;
	double	_centerExposition;
	int	_focusScanStart,_focusScanFinish;
	int	_centerScanStart,_centerScanFinish;
	int	_workZone;
	int	_centralZone;
	int	_maxIterCentering;
	double	_speedRA;
	double	_speedDec;
	double	_normalBaseTime;
	double	_generalBaseTime,_generalSecondBaseTime;
	double	_focusBaseTime;
	double	_centerBaseTime;

// Cn2 restor.: fi[X]ed, f[L]oating layers
	bool	_normalFixedMethod;
	bool	_generalFixedMethod;

// Polling
	double	_temperPollingTime;
	double	_buttonPollingTime;
	double	_highPollingTime;

// Buttons mode
	int	 	_aperturesMode;
	int		_illumMode;
	int		_shiftMode;
	int		_doorMode;


// Initial & current:
	int _inputDoor;
	int _initialStop;
	int _illumValue;
	int _illumination;
	int _buttons;
	int _buttonMode;
	int _highVoltage;
	int _hvLevel;
	int _safety;
	QArray<double> _discrLevels;

	double	_temperature;


// Parking:
	int _inputDoorPark;
	int _closeApert;
	int _buttonsPark;
	int _hvPark;
	int _illumPark;

// Tests:
	int _playInitialScenario; // On/Off

	double	_testVAmpl;

	int			_nStatisticTestLight;
	QArray<double>	_statisticTestLight;
	int			_nDetectorCountingLight;
	QArray<double>	_detectorCountingLight;
	int			_nDetectorCountingVoltage;
	QArray<int>	_detectorCountingVoltage;
	int			_nDetectorCountingDiscrimination;
	QArray<double>	_detectorCountingDiscrimination;
	int				_nDetectorStatisticLight;
	QArray<double>	_detectorStatisticLight;
	double			_detectorTestLight;	

	int		_showStarApert;
	int		_testApert;
	int		_normalApert,_generalApert,_generalSecondApert,_focusApert;
	int		_centerApert,_backgroundApert,_countApert,_statisticApert;

	int		_backgroundApShift;
	double		_backgroundTelShift;
	int		_backgroundMethod;

	double	_exchangeTestTime,_detectorTestTime;
	double	_detectorCountingTime,_detectorStatisticTime;
	double	_backgroundTime;
	double	_normalAccumTime;
	double	_generalAccumTime;
	double	_estimationAccumTime;
	double	_focusAccumTime;
	double	_centerAccumTime;
	double	_centerRefreshAccumTime;
	double	_fluxTolerance;
	double	_nonPoisTolerance;
	QArray<double>	_testCounts;

	QArray<double> _backgroundCorrectionCoefficients;
	QArray<double> _backgrounds;
	QArray<double> _nonPoisson;
	QArray<double> _nonLinearity;

// Optic & Site:
	double	_scale;
	double	_sinPosAngle;
	double	_cosPosAngle;
	int	_mirror;

	double _shiftRA;
	double _shiftDec;

	Time _startBaseTime;

// Scenario

	QArray<char>		currentScenario;
	QArray<QString*>	scenarioName;
	QArray<QString*>	scenarioScript;
	int	_nScenario;
	int	_currentScenarioNumber;
	int	_stopOnError;

	int	_stopScenario;
	bool	_pauseScenario;
	int	_scenarioStartPosition;

	matrix43 	*coef;
	PointDouble	*delta;

	int _nCorrected[MAX_N_COUNTERS];
// PlayBack
	long	massInFilePos;

private:

	void	startCounting( void );
	void	endCounting( void );
	void	disablePolling( void );
	void	enablePolling( void );
	void	startTemperPolling( void );
	void	stopTemperPolling( void );	
	void	startOverLightPolling( void );
	void	stopOverLightPolling( void );
	void	startButtonsPolling( void );
	void	stopButtonsPolling( void );
	void	setBrightness( int,int );
	void	changeBrightness( int );
	int		changeAperture( int, int );

	int		setExpos( double,int );
	int		setBlSize( int,int );
	int 		calcBlockNumber(double& baseTime, double exposition);
	int 		calcExpositionNumber( int _blockNumber );
	int		setBlockNumber( int,int&,int,bool isPlayBack=false );
	int		setDiscriminations( QArray<double>&, int );
	void		getDiscriminations( QArray<double>& );
	int		synchAt( int,int );
	int		shiftAt( int,int );
	int		shiftTelescope( int coordinate, // RA or DEC
					double shift, // time-sec
					int dialog );

	void		openProgressor( const QString& );	//
	void		closeProgressor( void );		//
	
	void		openMode( const QString&,double );	//
	void		closeMode( void );		//

	void		sendScenarioName( const QString& name );	//
	void		openScenario( const QString&,double );	//
	void		closeScenario( void );		//

	int		runNormal( FILE* mass_in_file=NULL );
	int		runGeneralized( FILE* mass_in_file=NULL );
	int		runPlayBackMode( FILE* mass_in_file, int mode );
	int		runFluxEstimation( void );
	void		calcApertShift(//double& shiftRA,
					//double& shiftDec,
					double shiftX,
					double shiftY );
	bool		isStarInWorkZone(void);
	bool		isStarInCentralZone(void);
	void		starInDiaphragm( void );
	int		scanAperture( double accumTime,
				double baseTime,
				int blockNumber,
				double exposition,
				int aperture,
				int scanStart,
				const QString& modeName,
				bool isFocus );
	int		runFocusControl( void );
	int		runCenterControl( void );
	int		runCentering( void );
	int		runExchangeTest( void );
	int		runDetectorTest( void );
	int		runStatisticTest( void );
	int		runDetectorStatisticMeasurement( void );	//!
	int		runDetectorCountingMeasurement( void );		//!
	int		runBackgroundCorrection( void );			//!
	int		runBackgroundMeasurement( void );			//!
	int		runCenterCorrection( void );				//!
	int		runTrackCalibration( void );

	void	terminateCountMode( int );		//!

public:
	DevManager( void );		//
	~DevManager();
	void	quitProgram( bool withPark=true );
	bool	withAutoGuide( void ) { return _withAutoGuide;}

	int	calcTimeoutInterval( double,int );
	void	startDevTimer( int );
	void	stopDevTimer( void );
	void	waitData( void );
	int	testUserStop( void );
	void	controlUserTrack( void );
	int	catchAppendix( void );
	void	outCountResult( void );

	int	resetCounters( int );
	void	resetDriverBlockCounter( void );

	bool	isCounterReady( void );
	void	create( void );		//
	int	testShutter( Counter* );	//
	int	testIllum( void );		//
	int	testButtons( void );	//
	int	testMotorStatus( int );	//
	int	testPowerStatus( int );	//
	int	setInduceRegime( int );	//
	int	resetTestCounter( void );	//
	int	switchTestCounter( void );	//
	Counter* getTestCounter( void ) { return( testCounter ); } //
	void 	resetTestData( void );	//
	int	runTestOneCounter( void );	//
	int	runTestAllCounters( void );	//
	int	runAllCounters( void );	//
	void	searchTestErrorsOneCounter( void );//
	void	searchTestErrorsAllCounters( void );//
	double	resumeTestOneCounter( void );	//
	double	resumeTestAllCounters( void );	//

	// Added for ver1.3: start and end-accum.Time 
	int	accumResume( Time midAccumTime, 
				Time startAccumTime, Time endAccumTime,
				bool isgen,
				bool isFixedLayersMethod,
				bool isPlayBack=false );
	// Added for ver1.3: start and end-accum.Time 
	int	makeAccumResultScind( Time midAccumTime,
				Time startAccumTime, Time endAccumTime,
				bool isgen );
	int	makeAccumResultAtmos( Time midAccumTime,
				bool isgen, bool isFixedLayersMethod );
	int	showAccumResultScind( bool isgen, bool isStatTest=false );
	void	showAccumResultAtmos( int row, bool isgen );

	void	prepareFiles( void );
	void	searchCountersSyncStatus( void );
	void	loadConst( void );

	void	putData( void );
	bool	cleanData( int& expositionNumber,
			long *addr = 0,
			bool isPlayBack = false );
	void	handleOverLight( void );

	void	setStartBaseTime( void );
	Time	setMidTime( Time startTime, Time stopTime );

	int	isIllumOn( void ) {return( light.isIllumOn() ); }
	int	isLightOn( void ) {return( light.isLightOn() ); }
	int	isHVOn( void );
	int	isButtonsOn( void ) {return( light.isButtonsOn() ); }
	int	isSafetyOn( void ) {return( power.isSafetyOn() ); }
	int	isDoorOpen( void ) {return( motor.isDoorOpen() ); }
	int	currentAperture( void ) { return( motor.currentAperture() ); }
	int	getCurrentBrightness( void ) { return( light.getCurrentBrightness() ); }
	int	setVoltage( int,int,bool _wait=true );
	int	getVoltage( void ) {return( power.getVoltage() );}
	int	getStopCorrect( void ) {return( motor.getStopCorrect() ); }
	int	getStopCorrectNew( void ) {return( motor.getStopCorrectNew() ); }
	int	getNStops( void ) { return( motor.getNStops() ); }
	int	getScenarioNumber( void ) {return( _currentScenarioNumber );}
	QString& getScenarioName( int number );
	int	getMaxScenarioNumber( void ) {return( _nScenario );}
	QString& getStopName( int );
	int	startShowStar( void );
	void	finishShowStar( int oldAperture );
	void	playBack( QString& fileName );
	void	endPlayBack( void );
	bool	isPlayBack( void ) {return _isPlayBack;}
	bool	isCounting( void ) {return _isCounting;}
	bool	isTracking( void ) {return _isTracking;}
	void	startAdjustAperture( void );
	void	acceptAdjustAperture( int );
	void	acceptBackgroundCorrection( void );
	void	acceptCenterCorrection( void );
	void	acceptTrackCalibration( void );
	void	endTrackCalibration( void );
	void	acceptComment( const QString& comment );
	void	getBackgroundCorrectionCoefficients( matrix43* );

	int	testOverLight( int );
	void	getTemper( int );
	void	processButtons( int );

	void	statistic_test_mode( void );
	void	detector_test_mode( void );
	void	exchange_test_mode( void );
	void	detector_counting_mode( void );
	void	detector_statistic_mode( void );
	void	background_corr_mode( void );
	void	background_measur_mode( void );
	void	centering_mode( void );
	void	centering_corr_mode( void );
	void	track_calibration_mode( void );
	void	focus_control_mode( void );
	void	center_control_mode( void );
	void	normal_mode( void );
	void	flux_estimation_mode( void );
	void	generalized_mode( void );
	double	getAccumTime( char mode );
	void	playback_mode( void );
	void	scenario_mode( void );

private:
	long	getCountAddress( FILE* mass_in_file,
			bool isgen,Time& startBaseTime );
	                                                

public slots:

	void	init( void );
	void	park( void );

	int	stopCounters( int );
	void	stopCounting( void );
	void	cleanUserStop( void );
	void	stopScenario( void );
	void	playScenario( void );
	void	setPause( bool isPause );
	void	change( void );
	int	turnDoor( const QString&, int, bool _backApert=true );
	int	turnDoor( int, int, bool _backApert=true );
	void	turnHV( int,int );
	int	turnIllum( const QString&, int );
	int	turnIllum( int, int );
	void	turnIllumOn( void );
	void	turnIllumOff( void );
	int	turnLight(int on, int dialog,
			int waitInterval=LIGHT_WAIT_INTERVAL );
	int	shiftAperture( int,int );
	int	setAperture( const QString&, int, bool _hard=false );
	int	setAperture( int, int, bool _hard=false );
	int	track( char coordinate, char direct );
	int	forceTrack(void);
	int	stopTrack(void);
	int	greatTrack( char coordinate, char direct,
				double trackTime ); // trackTime in sec
	int	trackToCenter( void );
//	void	calibrateTracking( void );
	void	setBrightnessUp( void );
	void	setBrightnessDown( void );
	int	setLight( double value, int dialog,
			int waitInterval = LIGHT_WAIT_INTERVAL );
	double	getLight( void ) {return( light.getLight() );}
	void	turnButtons( const QString&,int );
	void	turnButtons( int,int );
	void	turnSafety( const QString&,int );
	void	turnSafety( int,int );
	void	nullSet( void );
	int		fineShift( int );
	int		movePlus( void );
	int		moveMinus( void );

	void	exchangeTest( void );
	void	detectorTest( void );
	void	statisticTest( void );
	void	playInitialScenario( void );
	char	decodeMode( char mode );
	void	setCurrentScenario( int number );
	void	detectorCountingMeasurement( void );
	void	detectorStatisticMeasurement( void );
	void	testVAmpl( void );
	void	backgroundCorrection( matrix43* );
	void	backgroundMeasurement( void );
	void	fluxEstimation( void );
	void	centerCorrection( PointDouble* );
	void	trackCalibration( PointDouble* );
	void	normal( void );
	void	generalized( void );
	void	focusControl( void );
	void	centerControl( void );
	void	centering( PointDouble* );
	void	setProgress( double );
	void	setRemainTimeMode( double );
	void	setRemainTimeScenario( double );

};

#endif
