/*--------------------------------------------------------------------------*/
/** @file exchange.h
@brief  Low-level exchange implementation of the program with the device driver.

                  MASS project: TURBINA module file
                       header file for exchange.cpp

Implements the commands sending, data and signals receiving from the device via
the driver inserted into OS kernel.


 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/

#ifndef EXCHANGE_H
#define EXCHANGE_H

#include <qstring.h>
#include "counter.h"
#include "trbconst.h"

extern int errno;

#define HEAD_SIZE	5	// size of header of block (bytes)

#define DEVICE_WAIT_INTERVAL		5		// 5 millisec
#define COUNTER_WAIT_INTERVAL		25		// 25 millisec
#define APERTURE_DELAY				50		// 50 millisec
#define USER_TRACK_STEP				1000	// 1000 millisec
#define LIGHT_WAIT_INTERVAL			30000	// 30 sec
#define SHORT_LIGHT_WAIT_INTERVAL	5000	// 5 sec
#define N_REPEAT_COMMAND			3
#define VALUE_TOLERANCE				0.01	// max otklonenie ustanovlennogo
						// znacheniya ot ustanavlivayemogo ( hv,expos...) 
#define POSITION_CLOSE				3100	// Abs position of motor ("DoorClose")

// Signals:

// from device:

#define ACK	0x87	// Successful receiving of data
#define NAK	0x96	// Damaged packed was received 
#define NOD	0xA5	// Required data is not ready  
#define ACN	0xB4	// Command is successfully recived but can't be executed
#define ACY	0xC3	// Command is successfully recived and executed
#define ACW	0xD2	// Command is successfully recived and will be executed

// from driver:

#define	NDV	0x82	// EHOSTDOWN	: no LPT converter detected
#define NOP 0x83	// ESHUTDOWN	: MASS is off or disconnected from RS485
#define DSC	0x84	// ETIMEDOUT
#define BUS	0x85	// EBUSY
#define COL	0x86	// ECONNREFUSED

#define MSG_SIZE	40

namespace exch {
	void	wait( int msec );
	int		sendCommand( const char*,
					bool save=true );
	int		sendCommand( char address,
					char command, 
					unsigned char** r,
					int buf_len=2,
					char low=0,
					char high=0,
					bool save=true );
	QString	talk( const QString& text );
	void 	writeToProc( char* to_proc_file, int size );
	void	resetBlockCounter( void );
	void	*read_data( void );

	void	initExchange( void );
	void	setCounters( Counter** p_counters );
	void	setSrvFile( FILE* f );
	int		setBlockSize( int size );
	void	setBlockNumber( int number );
	int		getNblock( void );
	void	setNcounters( int number );
	int		getBlockNumber( void );
	int		getNreadData( void );
	void	setTimeout( bool tmout );
	bool	isTimeout( void );
	void	timeoutAction( void );
	void	waitData( void );
	void	putData( void );
	        
	void	startCounting( void );
	void	quitDataCycle( void );
	void	terminate_program( void );

	bool	withOldDriver(void);
} // end of namespace

#endif