#define QT_THREAD_SUPPORT
#include <qthread.h>
#include <qapplication.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qtextstream.h>
#include <qtooltip.h>
#include <qstatusbar.h>
#include <qmenubar.h>
#include <qmessagebox.h>

#include "mywidget.h"
#include "tree.h"
#include "trbexport.h"
#include "trbtime.h"
#include "messages.h"
#include "tools.h"
#include "cfg.h"
#include "device.h"
#include "star.h"
#include "names.h"
#include "diaphragm.xpm"
#include "buttons.xpm"

extern DevManager	devmanager;
extern Star		star;
extern CFGTree*		tree;
extern QMutex		infoExportMutex;
extern bool		isMassRoot;
extern bool		withDevice;

static const char* Undefin	= "?";
static const char* Open		= "Open";
static const char* Close	= "Close";
static const char* On		= "On";
static const char* Off		= "Off";

// ToolTips:
static const char* diaphragmText	= "Star in diaphragm";
static const char* raPlusText		= "RA+";
static const char* raMinusText		= "RA-";
static const char* decPlusText		= "Dec+";
static const char* decMinusText		= "Dec-";
static const char* stopText 		= "Stop Scenario";
static const char* pauseText		= "Pause";
static const char* playText		= "Press to run scenario";
static const char* navigateText		= "Press to show star info";
static const char* selectText		= "Press to select new star";  
static const char* apertText		= "Current Aperture";
static const char* illumText		= "Illumination";
static const char* doorText		= "Input Door";
static const char* hvText		= "High Voltage";
static const char* tmprText		= "Temperature, C";
static const char* starText		= "Observed Star";
static const char* zenitText		= "Air Mass";
static const char* scenarioText		= "Current Scenario";
static const char* modeText		= "Current Mode";
static const char* modeTimeText		= "Mode Remained Time";
static const char* scenarioTimeText	= "Scenario Remained Time";
static const char* clockText		= "Local Time";

void MyWidget::setLabelText( QLabel *label, const QString& text ) {
	if( label->text() != text ) {
		int width = fontMetrics().width( text );
		label->setText( text );
		if( label->width() < width ) {
			label->setFixedSize(label->sizeHint() );
		}
	}
}

int MyWidget::addItem( QMenuData* menuData,
				const char* helpText,
				bool withDeviceEnable = true,
				bool forRootEnable = false,
				bool countDisable = true ) {
	int ret = item_id;
	if( menu_help.size() <= (uint)item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( helpText );
	if( withDeviceEnable ) {
		if( device_enable.size() <= (uint)n_dev ) 
				device_enable.resize(device_enable.size()+10);
		device_enable[n_dev] = item_id;
		n_dev++;
	}
	bool deviceEnable = !withDeviceEnable || withDevice;
	bool rootEnable = !forRootEnable || isMassRoot;
	bool enable = deviceEnable && rootEnable;
	menuData->setItemEnabled( item_id,enable );
	if( countDisable && enable ) {
		if( disabl_count.size() <= (uint)n_dis ) 
				disabl_count.resize(disabl_count.size()+10);
		disabl_count[n_dis] = item_id;
		n_dis++;
	}
	item_id++;
	return ret;
}


MyWidget::MyWidget( QWidget *parent, const char *name )
					: QMainWindow( parent, name )	{
	isTimerEventLocked = false;
	isScenario = false;
	oldStarPosition = -1;
	existTalkDialog = false;
	existAdjustDialog = false;
	existIllumDialog = false;
	existLightDialog = false;
	existHVDialog = false;
	existSelectStarDialog = false;
	existShowStarDialog = false;
	existCommentDialog = false;
	existCenteringDialog = false;
	existBackCorrectionDialog = false;
	existCenterCorrectionDialog = false;
	existTrackCalibrationDialog = false;
	existHelpView = false;
	_updateHA = true;
//	_disableKeyEvent = false;
	mainView = NULL;
	cnfg = NULL;

	mainView = new QJKMainView( this,"MainView" );
	CHECK_PTR( mainView );

	loadConst();

	mainView->setFont( QFont("Helvetica",_fontSizeSmall),true );
	mainView->fitToFont();
	setCentralWidget( mainView );

/*
	progressor = new QProgressBar( 100,statusBar() );
	CHECK_PTR( progressor );
	statusBar()->addWidget( progressor,0,true );
	progressor->hide();
*/
	toolBarDevice = new QToolBar( this );
	CHECK_PTR( toolBarDevice );
	addToolBar( toolBarDevice,"Device status",Bottom,true );
	toolBarDevice->setHorizontalStretchable( true );

	toolBarObject = new QToolBar( this );
	CHECK_PTR( toolBarObject );
	addToolBar( toolBarObject,"Object status",Top,true );
	toolBarObject->setHorizontalStretchable( true );

	toolBarProgram = new QToolBar( this );
	CHECK_PTR( toolBarProgram );
	addToolBar( toolBarProgram,"Program status",Top,true );
	toolBarProgram->setHorizontalStretchable( true );

	toolBarButtons = new QToolBar( this );
	CHECK_PTR( toolBarButtons );
	addToolBar( toolBarButtons,"",Left,true );
	toolBarButtons->setVerticalStretchable( true );

	labelApert = new QLabel( "",toolBarDevice );
	CHECK_PTR( labelApert );
	labelApert->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelApert->setAlignment( AlignCenter );
	labelApert->setFixedSize( labelApert->sizeHint() );
	QToolTip::add( labelApert,apertText );

	labelDoor = new QLabel( "",toolBarDevice );
	CHECK_PTR( labelDoor );
	labelDoor->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelDoor->setAlignment( AlignCenter );
	labelDoor->setFixedSize( labelDoor->sizeHint() );
	QToolTip::add( labelDoor,doorText );

	labelIllum = new QLabel( "",toolBarDevice );
	CHECK_PTR( labelIllum );
	labelIllum->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelIllum->setAlignment( AlignCenter );
	labelIllum->setFixedSize( labelIllum->sizeHint() );
	QToolTip::add( labelIllum,illumText );

	labelHV = new QLabel( "",toolBarDevice );
	CHECK_PTR( labelHV );
	labelHV->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelHV->setAlignment( AlignCenter );
	labelHV->setFixedSize( labelHV->sizeHint() );
	QToolTip::add( labelHV,hvText );

	labelTmpr = new QLabel( "-99.9",toolBarDevice );
	CHECK_PTR( labelTmpr );
	labelTmpr->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelTmpr->setAlignment( AlignCenter );
	labelTmpr->setFixedSize( labelTmpr->sizeHint() );
	QToolTip::add( labelTmpr,tmprText );
	toolBarDevice->setStretchableWidget( labelTmpr );

	toolBarDevice->addSeparator();

	labelDiaphragm = new QLabel( toolBarObject );
	CHECK_PTR( labelDiaphragm );
	labelDiaphragm->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelDiaphragm->setAlignment( AlignCenter );
	QToolTip::add( labelDiaphragm, diaphragmText );
	displayStarPosition(-1);

	labelStar = new QLabel( "",toolBarObject );
	CHECK_PTR( labelStar );
	labelStar->setText( star.getName() );
	labelStar->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelStar->setAlignment( AlignCenter );
	labelStar->setFixedSize( labelStar->sizeHint() );
	QToolTip::add( labelStar,starText );
	toolBarObject->setStretchableWidget(labelStar);

	labelZenit = new QLabel( "99.99",toolBarObject );
	CHECK_PTR( labelZenit );
	labelZenit->setText( star.getAirMass() );
	labelZenit->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelZenit->setAlignment( AlignCenter );
	labelZenit->setFixedSize( labelZenit->sizeHint() );
	QToolTip::add( labelZenit,zenitText );

	digitalclock = new DigitalClock( toolBarObject );
	CHECK_PTR( digitalclock );
	digitalclock->setFrameStyle( QFrame::Panel | QFrame::Sunken );
//	digitalclock->setFixedSize( digitalclock->sizeHint() );
// Konstructor stavit size
	QToolTip::add( digitalclock,clockText );
	toolBarObject->addSeparator();

	labelMode = new QLabel( "",toolBarProgram );
	CHECK_PTR( labelMode );
	labelMode->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelMode->setAlignment( AlignCenter );
	labelMode->setFixedSize( labelMode->sizeHint() );
	QToolTip::add( labelMode,modeText );

	labelModeTime = new QLabel( "",toolBarProgram );
	CHECK_PTR( labelModeTime );
	labelModeTime->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelModeTime->setAlignment( AlignCenter );
	labelModeTime->setFixedSize( labelModeTime->sizeHint() );
	QToolTip::add( labelModeTime,modeTimeText );

	labelScenario = new QLabel( devmanager.getScenarioName(
				devmanager.getScenarioNumber() ),
				toolBarProgram );
	CHECK_PTR( labelScenario );
	labelScenario->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelScenario->setAlignment( AlignCenter );
	labelScenario->setFixedSize( labelScenario->sizeHint() );
	QToolTip::add( labelScenario,scenarioText );	

	labelScenarioTime = new QLabel( "",toolBarProgram );
	CHECK_PTR( labelScenarioTime );
	labelScenarioTime->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	labelScenarioTime->setAlignment( AlignCenter );
	labelScenarioTime->setFixedSize( labelScenarioTime->sizeHint() );
	QToolTip::add( labelScenarioTime,scenarioTimeText );

	toolBarProgram->setStretchableWidget(labelScenario);

	QIconSet navigateIconSet,selectStarIconSet,playIconSet;
	QIconSet stopIconSet,pauseIconSet;
	QIconSet raPlusIconSet,raMinusIconSet,decPlusIconSet,decMinusIconSet;
	navigateIconSet.setPixmap( navigate,QIconSet::Automatic,QIconSet::Normal );
	navigateIconSet.setPixmap( navigateDisabled,QIconSet::Automatic,QIconSet::Disabled );

	selectStarIconSet.setPixmap( selectStar,QIconSet::Automatic,QIconSet::Normal );
	selectStarIconSet.setPixmap( selectStarDisabled,QIconSet::Automatic,QIconSet::Disabled );

	playIconSet.setPixmap( play,QIconSet::Automatic,QIconSet::Normal );
	playIconSet.setPixmap( playDisabled,QIconSet::Automatic,QIconSet::Disabled );

	stopIconSet.setPixmap( setStop,QIconSet::Automatic,QIconSet::Normal );
	stopIconSet.setPixmap( setStopDisabled,QIconSet::Automatic,QIconSet::Disabled );

	pauseIconSet.setPixmap( setPause,QIconSet::Automatic,QIconSet::Normal );
	pauseIconSet.setPixmap( setPauseDisabled,QIconSet::Automatic,QIconSet::Disabled );

	raPlusIconSet.setPixmap( raPlusIcon,QIconSet::Automatic,QIconSet::Normal );
	raPlusIconSet.setPixmap( raPlusDisabled,QIconSet::Automatic,QIconSet::Disabled );

	raMinusIconSet.setPixmap( raMinusIcon,QIconSet::Automatic,QIconSet::Normal );
	raMinusIconSet.setPixmap( raMinusDisabled,QIconSet::Automatic,QIconSet::Disabled );

	decPlusIconSet.setPixmap( decPlusIcon,QIconSet::Automatic,QIconSet::Normal );
	decPlusIconSet.setPixmap( decPlusDisabled,QIconSet::Automatic,QIconSet::Disabled );

	decMinusIconSet.setPixmap( decMinusIcon,QIconSet::Automatic,QIconSet::Normal );
	decMinusIconSet.setPixmap( decMinusDisabled,QIconSet::Automatic,QIconSet::Disabled );

	navigateButton = new QToolButton( navigateIconSet,navigateText,"",
		this,SLOT( slotShowStarDialog() ),toolBarButtons,"NavigateButton" );
	CHECK_PTR(navigateButton);
	navigateButton->setUsesBigPixmap( true );
	navigateButton->setAutoRaise( false );

	selectButton = new QToolButton( selectStarIconSet,selectText,"",
		this,SLOT( slotSelectStarDialog() ),toolBarButtons,"SelectButton" );
	CHECK_PTR(selectButton);
	QToolTip::add( selectButton,selectText );
	selectButton->setUsesBigPixmap( true );
	selectButton->setAutoRaise( false );

	playButtonGroup = new QButtonGroup( this );
	CHECK_PTR( playButtonGroup );
	playButtonGroup->setExclusive( true );

	playButton = new QToolButton( playIconSet, playText,"",
		this, SLOT( playScenario() ),toolBarButtons,"PlayButton");
	CHECK_PTR(playButton);
	playButtonGroup->insert( playButton,1 );
	playButton->setToggleButton( true );
	playButton->setUsesBigPixmap( true );
	playButton->setAutoRaise( false );
	playButton->setDisabled( !withDevice );

	stopButton = new QToolButton( stopIconSet, stopText, "",
		this, SLOT( stopScenario() ), toolBarButtons,"StopButton" );
	CHECK_PTR( stopButton );
	playButtonGroup->insert( stopButton,2 );
	stopButton->setToggleButton( true );
	stopButton->setUsesBigPixmap( true );
	stopButton->setAutoRaise( false );
	stopButton->setDisabled( true );

	pauseButton = new QToolButton( pauseIconSet, pauseText,"",
		this, SLOT( togglePause() ), toolBarButtons ,"PauseButton");
	CHECK_PTR(pauseButton);
	pauseButton->setUsesBigPixmap( true );
	pauseButton->setAutoRaise( false );
	pauseButton->setToggleButton( true );
	pauseButton->setDisabled( true );

	labelTrack = new QLabel( "",toolBarButtons );
	CHECK_PTR( labelTrack );
	toolBarButtons->setStretchableWidget(labelTrack);

	raPlus = new QToolButton( toolBarButtons,"RAplusButton" );
	CHECK_PTR(raPlus);
	raPlus->setIconSet(raPlusIconSet);
	raPlus->setTextLabel(raPlusText);
	raPlus->setUsesBigPixmap(true);
	raPlus->setAutoRaise(false);
	if( !devmanager.withAutoGuide() ) raPlus->setDisabled(true);
	connect( raPlus, SIGNAL(pressed()), this, SLOT(moveRAplus()) );
	connect( raPlus, SIGNAL(released()),
			this, SLOT(stopTrack()) );

	raMinus = new QToolButton( toolBarButtons,"RAplusButton" );
	CHECK_PTR(raMinus);
	raMinus->setIconSet(raMinusIconSet);
	raMinus->setTextLabel(raMinusText);
	raMinus->setUsesBigPixmap(true);
	raMinus->setAutoRaise(false);
	if( !devmanager.withAutoGuide() ) raMinus->setDisabled(true);
	connect( raMinus, SIGNAL(pressed()), this, SLOT(moveRAminus()) );
	connect( raMinus, SIGNAL(released()),
			this, SLOT(stopTrack()) );

	decPlus = new QToolButton( toolBarButtons,"DecPlusButton" );
	CHECK_PTR(decPlus);
	decPlus->setIconSet(decPlusIconSet);
	decPlus->setTextLabel(decPlusText);
	decPlus->setUsesBigPixmap(true);
	decPlus->setAutoRaise(false);
	if( !devmanager.withAutoGuide() ) decPlus->setDisabled(true);
	connect( decPlus, SIGNAL(pressed()), this, SLOT(moveDecPlus()) );
	connect( decPlus, SIGNAL(released()),
			this, SLOT(stopTrack()) );

	decMinus = new QToolButton( toolBarButtons,"DecMinusButton" );
	CHECK_PTR(decMinus);
	decMinus->setIconSet(decMinusIconSet);
	decMinus->setTextLabel(decMinusText);
	decMinus->setUsesBigPixmap(true);
	decMinus->setAutoRaise(false);
	if( !devmanager.withAutoGuide() ) decMinus->setDisabled(true);
	connect( decMinus, SIGNAL(pressed()), this, SLOT(moveDecMinus()) );
	connect( decMinus, SIGNAL(released()),
			this, SLOT(stopTrack()) );

	toolBarProgram->addSeparator();

	item_id = 0;
	menu_help.resize(100);
	n_dis = 0;
	disabl_count.resize(100);
	n_dev = 0;
	device_enable.resize(100);

	file = new QPopupMenu;
	CHECK_PTR( file );

	file->insertItem( "Comment...",
		this,SLOT( slotCommentDialog() ),ALT+Key_C,item_id );
	addItem( file,"Comment",false,false,false );

	file->insertItem( "Select star...",
		this,SLOT( slotSelectStarDialog() ),0,item_id );
	addItem( file,"Select star",false,false,false );

	file->insertItem( "Show star...",
		this,SLOT( slotShowStarDialog() ),0,item_id );
	addItem( file,"Show star",false );

	file->insertItem( "Play Back", this, SLOT(slotPlayBack()),0,item_id );
	pb_id = addItem( file,"Play Back",false,false,false );

	file->insertSeparator();

	file->insertItem( "Exit", this, SLOT(myQuit()),CTRL+Key_Q,item_id );
	addItem( file,"Exit",false,false,true );

	help = new QPopupMenu;
	CHECK_PTR( help );
	help->insertItem( "Help", this, SLOT(slotHelp()),CTRL+Key_H,item_id );
	addItem( help,"Help",false,false,false );

	measure = new QPopupMenu;
	CHECK_PTR( measure );

	measure->insertItem( "Normal", &devmanager, SLOT(normal()),
						0, item_id );
	addItem( measure,"Normal" );

	measure->insertItem( "Generalized", &devmanager, SLOT(generalized()),
						0, item_id );
	addItem( measure,"Generalized" );


	measure->insertItem( "Center control", &devmanager,
				SLOT(centerControl()), 0, item_id );
	addItem( measure,"Star centering control" );

	measure->insertItem( "Focus control",
				&devmanager,SLOT(focusControl()),
						0, item_id );
	addItem( measure,"Focus control" );

	measure->insertItem( "Background",
			&devmanager,SLOT(backgroundMeasurement()),
							0, item_id );
	addItem( measure,"Background measurement" );

	measure->insertItem( "Flux Estimation",&devmanager,SLOT(fluxEstimation()),
						CTRL+Key_F,item_id );
	addItem( measure,"Flux Estimation" );

	measure->insertSeparator();

	scenario_set = new QPopupMenu;
	CHECK_PTR( scenario_set );
	measure->insertItem( "Select Scenario",scenario_set,
				item_id );
	addItem( measure,"Select Scenario" );

 	int nscen = devmanager.getMaxScenarioNumber();;
	scen_id.resize( nscen );
	for( int i=0; i<nscen; i++ ) {
		scenario_set->insertItem( devmanager.getScenarioName(i),
				item_id );
		scen_id[i] = addItem( scenario_set,
				devmanager.getScenarioName(i).data(),
				true,false,false );
	}
	connect( scenario_set, SIGNAL(activated(int)),
		this, SLOT(slotSetScenario(int)) );

	measure->insertSeparator();

	measure->insertItem( "Run scenario",
				this,SLOT(playScenario()),0,item_id );
	play_item = addItem( measure,playText,true,false,false );

	measure->insertItem( "Stop scenario",
				this,SLOT(stopScenario()),0,item_id );
	stop_item = addItem( measure,stopText,true,false,false );
	measure->setItemEnabled( stop_item, false );

	measure->insertItem( "Pause scenario",
				this,SLOT(togglePauseMenu()),0,item_id );
	pause_item = addItem( measure,pauseText,true,false,false );
	measure->setItemEnabled( pause_item, false );

 	tests = new QPopupMenu;
	CHECK_PTR( tests );

	tests->insertItem( "Exchange Test",&devmanager,SLOT(exchangeTest()),
						0,item_id );
	addItem( tests,"Run full exchange test" );

	tests->insertItem( "Detectors Test",&devmanager,SLOT(detectorTest()),
						0,item_id );
	addItem( tests,"Run detectors test" );

	tests->insertItem( "Statistic Test",&devmanager,SLOT(statisticTest()),
						0,item_id );
	addItem( tests,"Run statistic test" );

	tools = new QPopupMenu;
	CHECK_PTR( tools );

	tools->insertItem( "Initialization", &devmanager,
				SLOT(init()),0,item_id );
	int id = addItem( tools,"Initialization" );

	tools->insertItem( "Detectors Counting measurement",&devmanager,SLOT(detectorCountingMeasurement()),
						0,item_id );
	addItem( tools,"Detectors Counting measurement..." );

	tools->insertItem( "Detectors Statistic measurement",&devmanager,SLOT(detectorStatisticMeasurement()),
						0,item_id );
	addItem( tools,"Detectors Statistic measurement..." );

	tools->insertItem( "Centering...",
		this,SLOT( slotCenteringDialog() ),0,item_id );
	addItem( tools,"Star centering" );

	tools->insertItem( "Track calibration...",
		this, SLOT(slotTrackCalibrationDialog()),0,item_id );
	id = addItem( tools,"Track calibration" );
	if( !devmanager.withAutoGuide() ) tools->setItemEnabled(id,false);

	tools->insertItem( "Background correction...",
		this,SLOT(slotBackCorrectionDialog()),
						0, item_id );
	addItem( tools,"Background correction" );

	tools->insertItem( "Aperture Center...",
			this,SLOT(slotCenterCorrectionDialog()),
							0,item_id );
	addItem( tools,"Fixation of aperture center" );

	tools->insertItem( "Apertures Adjust...",
		this,SLOT( slotAdjustDialog() ),0,item_id );
	addItem( tools,"Apertures Adjust" );

	tools->insertSeparator();

	tools->insertItem( "Talk",this,SLOT(slotTalkDialog()),0,item_id );
	id = addItem( tools,"Talk (debug mode)",true,true,false );

	tools->insertItem( "Break Accumulation",&devmanager,SLOT(stopCounting()),
						CTRL+Key_S,item_id );
	break_item = addItem( tools,"Break Accumulation",true,false,false );
	tools->setItemEnabled( break_item, false );

	tools->insertItem( "Parking", &devmanager,
			SLOT(park()),0,item_id );
	addItem( tools,"Parking" );

	functions = new QPopupMenu;
	CHECK_PTR( functions );

	functions->insertItem( "",
		this, SLOT( turnDoor() ),0,item_id );
	door_id = addItem( functions,"Door Open/Close" );
	slotTurnDoor(-1);

	functions->insertItem( "",
			this,SLOT( turnHV() ),0,item_id );
	hv_id = addItem( functions,"HV On/Off" );
	slotTurnHV(-1);

	functions->insertItem( "",
		this, SLOT( turnSafety() ),0,item_id );
	safety_id = addItem( functions,"OverLight safety On/Off",true,true );
	slotTurnSafety(-1);

	functions->insertItem( "",
		this,SLOT( turnButtons() ),0,item_id );
	buttons_id = addItem( functions,"Buttons On/Off" );
	slotTurnButtons(-1);

	functions->insertItem( "",
		this, SLOT( turnIllum() ),0,item_id );
	illum_id = addItem( functions,"Illumintaion On/Off" );
	slotTurnIllum(-1);

	functions->insertItem( "",
		this, SLOT( turnLight() ),0,item_id );
	light_id = addItem( functions,"Control Light On/Off",true,true );
	slotTurnLight(-1);

	functions->insertSeparator();

	functions->insertItem( "Set Illumination level...",
		this,SLOT( slotIllumDialog() ),0,item_id );
	id = addItem( functions,"Set Illumination level" );

	functions->insertItem( "Set Light level...",
		this,SLOT( slotLightDialog() ),0,item_id );
	id = addItem( functions,"Set Light level",true,true );

	functions->insertItem( "Set High Voltage...",
		this,SLOT( slotHVDialog() ),0,item_id );
	id = addItem( functions,"Set High Voltage",true,true );

	aperture_set = new QPopupMenu;
	CHECK_PTR( aperture_set );
	functions->insertItem( "Set &Aperture",aperture_set,
				item_id );
	id = addItem( functions,"Set Aperture" );

	int ns = devmanager.getNStops();
	apert_id.resize( ns );
	for( int i=0; i<ns; i++ ) {
		aperture_set->insertItem( devmanager.getStopName(i),
			item_id );
		apert_id[i] = addItem( aperture_set,devmanager.getStopName(i).data() );
	}
	slotAperture( devmanager.currentAperture() );
	connect( aperture_set, SIGNAL(activated(int)),
		this, SLOT(setAperture(int)) );

	functions->insertItem( "Set Null-Point",
		&devmanager,SLOT( nullSet() ),0,item_id );
	id = addItem( functions,"Set Null-Point" );

	config = new QPopupMenu;
	CHECK_PTR( config );
	cnfg = new CFGDialog( config,&item_id,menu_help );

	menuBar()->insertItem( "&File",file );
	menuBar()->insertItem( "&Measurement",measure );
	menuBar()->insertItem( "T&est",tests );
	menuBar()->insertItem( "&Tools",tools );
	menuBar()->insertItem( "&Control",functions );
	menuBar()->insertItem( "Confi&g",config );
	menuBar()->insertSeparator();
	menuBar()->insertItem( "&Help",help );

	connect( menuBar(),SIGNAL(highlighted(int)),
				this,SLOT(slotHighlighted(int)) );
	connect( cnfg,SIGNAL(signalShowStatus(const char*,int)),
				this,SLOT(slotShowMessage(const char*,int)) );
	connect( tree, SIGNAL( changed() ), this,SLOT( change() ) );


	normalTimer = startTimer( 1000 );	// 1 sec

	setGeometry( _winX, _winY, _winWidth, _winHeight );
	statusBar()->message( "Ready",2000 );
	repaint();
}

void MyWidget::myQuit( void ) {
	jdebug( "mywidget::myQuit..." );
	if( cnfg ) cnfg->testCfgChang();
	devmanager.quitProgram();
	qApp->quit();
}

void MyWidget::playScenario( void ) {
	devmanager.playScenario();
//	showPlayScenario();
}

void MyWidget::enableSelectScenario( bool enable ) {
	for( uint i=0; i<scen_id.size(); i++ ) {
		menuBar()->setItemEnabled( scen_id[i],enable );
	}
}

void MyWidget::showPlayScenario( void ) {
	playButton->setOn( true );
	playButton->setDisabled( true );
	menuBar()->setItemEnabled( play_item, false );
	enableSelectScenario( false );

	stopButton->setOn( false );
	stopButton->setDisabled( false );

	menuBar()->setItemEnabled( stop_item, true );
	pauseButton->setDisabled( false );
	menuBar()->setItemEnabled( pause_item, true );
}

void MyWidget::stopScenario( void ) {
	devmanager.stopScenario();
//	showStopScenario();
}

void MyWidget::showStopScenario( void ) {
	playButton->setOn( false );
	playButton->setDisabled( false );
	menuBar()->setItemEnabled( play_item, true );
	enableSelectScenario( true );

	stopButton->setOn( true );
	stopButton->setDisabled( true );

	menuBar()->setItemEnabled( stop_item, false );
	pauseButton->setDisabled( true );
	menuBar()->setItemEnabled( pause_item, false );
}

void MyWidget::togglePauseMenu( void ) {
	pauseButton->toggle();
	togglePause();
}

void MyWidget::togglePause( void ) {
	jdebug( "&&&&&&&&&&&&&&&&&&&&&&&&&&&&& toggle Pause!" );
	if( pauseButton->isOn() ) {
		jdebug( "&&&&&&&&&& pauseOn!" );
		stopButton->setDisabled( true );
		menuBar()->setItemEnabled( stop_item, false );
		devmanager.setPause(true);
	} else {
		jdebug( "&&&&&&&&&& pauseOff!" );
		stopButton->setDisabled( false );
		menuBar()->setItemEnabled( stop_item, true );
		devmanager.setPause(false);
	}
	jdebug( "&&&&&&&&&& end toggle" );
}

void MyWidget::slotTurnIllum( int _on ) {
	jdebug( "Illum On/off: %d",_on );
	switch ( _on ) {
		case  ON : functions->changeItem( illum_id, 
			"Turn Illumintaion\tOff" );
			setLabelText( labelIllum, On );
			break;
		case  OFF : functions->changeItem( illum_id,
			"Turn Illumintaion\tOn" );
			setLabelText( labelIllum,Off );
			break;
		case -1 : functions->changeItem( illum_id,
			"Turn Illumintaion\tOn/Off" );
			setLabelText( labelIllum,Undefin );
			break;
	}
}

void MyWidget::slotTurnSafety( int _on ) {
	jdebug( "Safety On/off: %d",_on );
	switch ( _on ) {
		case  ON : functions->changeItem( safety_id, 
			"Turn Safety\tOff" );
			break;
		case  OFF : functions->changeItem( safety_id,
			"Turn Safety\tOn" );
			break;
		case -1 : functions->changeItem( safety_id,
			"Turn Safety\tOn/Off" );
			break;
	}
}

void MyWidget::slotTurnLight( int _on ) {
	jdebug( "Light On/off: %d",_on );
	switch ( _on ) {
		case  ON : functions->changeItem( light_id, 
			"Turn Light\tOff" );
			break;
		case  OFF : functions->changeItem( light_id,
			"Turn Light\tOn" );
			break;
		case -1 : functions->changeItem( light_id,
			"Turn Light\tOn/Off" );
			break;
	}
}

void MyWidget::slotTemperature( double _temperature ) {
	QString str;
	str.setNum( _temperature,'f',1 );
	setLabelText( labelTmpr, str );
}

int MyWidget::moveRAplus( void ) {
	int success = 1;
	if( devmanager.isCounting() || isScenario )
		StateExport::startUserTrack(RA,1);
	else
		if( !(success=devmanager.track(RA,1)) )
			raPlus->setDown(false);
	return success;
}

int MyWidget::moveRAminus( void ) {
	int success = 1;
	if( devmanager.isCounting() || isScenario )
		StateExport::startUserTrack(RA,-1);
	else
		if( !(success=devmanager.track(RA,-1)) )
			raMinus->setDown(false);
	return success;
}

int MyWidget::moveDecPlus( void ) {
	int success = 1;
	if( devmanager.isCounting() || isScenario )
		StateExport::startUserTrack(DECLINATION,1);
	else
		if( !(success=devmanager.track(DECLINATION,1)) )
			decPlus->setDown(false);
	return success;
}

int MyWidget::moveDecMinus( void ) {
	int success = 1;
	if( devmanager.isCounting() || isScenario )
		StateExport::startUserTrack(DECLINATION,-1);
	else
		if( !(success=devmanager.track(DECLINATION,-1)) )
			decMinus->setDown(false);
	return success;
}

void MyWidget::stopTrack( void ) {
	StateExport::stopUserTrack();
	if( !devmanager.isCounting() && !isScenario )
		devmanager.stopTrack();
}

void MyWidget::turnIllum( void ) {
	if( devmanager.isIllumOn() ) {
		devmanager.turnIllum(OFF,1);
	} else {
		devmanager.turnIllum(ON,1);
	}
}

void MyWidget::turnSafety( void ) {
	if( devmanager.isSafetyOn() ) {
		devmanager.turnSafety(OFF,1);
	} else {
		devmanager.turnSafety(ON,1);
	}
}

void MyWidget::turnLight( void ) {
	if( devmanager.isLightOn() ) {
		devmanager.turnLight(OFF,1);
	} else {
		devmanager.turnLight(ON,1,0);
	}
}

void MyWidget::slotTurnHV( int _on ) {
	jdebug( "HV On/off: %d",_on );
	switch( _on ) {
		case  ON : functions->changeItem( hv_id,
			"Turn HV\tOff" );
			setLabelText( labelHV, On );
			break;
		case  OFF : functions->changeItem( hv_id,
			"Turn HV\tOn" );
			setLabelText( labelHV, Off );
			break;
		case -1 : functions->changeItem( hv_id,
			"Turn HV\tOn/Off" );
			setLabelText( labelHV, Undefin );
			break;
	}
}

void MyWidget::turnHV( void ) {
	int _on = devmanager.isHVOn();
	if( _on == ON ) {
		devmanager.turnHV(OFF,1);
	} else {
		devmanager.turnHV(ON,1);
	}
}

void MyWidget::slotTurnButtons( int _on ) {
	jdebug( "Buttons On/off: %d",_on );
	switch( _on ) {
		case  1 : functions->changeItem( buttons_id,
			"Buttons\tLock" ); break;
		case  0 : functions->changeItem( buttons_id,
			"Buttons\tUnLock" ); break;
		case -1 : functions->changeItem( buttons_id,
			"Buttons\tLock/UnLock" ); break;
	}
}

void MyWidget::turnButtons( void ) {
	if( devmanager.isButtonsOn() ) {
		devmanager.turnButtons(0,1);
	} else {
		devmanager.turnButtons(1,1);
	}
}

void MyWidget::slotTurnDoor( int _open ) {
	jdebug( "Set Door On/off: %d",_open );
	switch( _open ) {
		case  OPEN : functions->changeItem( door_id,
			"Door\tClose" ); 
			setLabelText( labelDoor, Open );
			break;
		case  CLOSE : functions->changeItem( door_id,
			"Door\tOpen" );
			setLabelText( labelDoor, Close );
			break;
		case -1 : functions->changeItem( door_id,
			"Door\tOpen/Close" );
			setLabelText( labelDoor, Undefin );
			break;
	}
}

void MyWidget::turnDoor( void ) {
	if( devmanager.isDoorOpen() == OPEN ) {
		devmanager.turnDoor(CLOSE,1);
	} else {
		devmanager.turnDoor(OPEN,1);
	}
}

void MyWidget::slotSetScenario( int _id ) {
	for( uint i=0; i<scen_id.size(); i++ ) {
		if( _id == scen_id[i] ) {
			jdebug( "MyWidget: set Scenario %d",i );
			devmanager.setCurrentScenario(i);
			return;
		}
	}
}

void MyWidget::slotAperture( int _stopNumber ) {

	setLabelText( labelApert, devmanager.getStopName(_stopNumber) );

	for( uint i=0; i<apert_id.size(); i++ ) {
		menuBar()->setItemEnabled( apert_id[i], true );
	}
	if( (_stopNumber < 0) || 
		(_stopNumber >= (int)apert_id.size() ) ) return;
	menuBar()->setItemEnabled( apert_id[_stopNumber], false );
}

void MyWidget::setAperture( int _id ) {
	for( uint i=0; i<apert_id.size(); i++ ) {
		if( _id == apert_id[i] ) {
			jdebug( "MyWidget: set Aperture %d",i );
			devmanager.setAperture(i,1);
			jdebug( "MyWidget::setAperture return" );
			return;
		}
	}
}

void MyWidget::setProgress( const QString& _text, double progress ) {
	if( progress == -1 ) openProgressor( _text );
	else if( progress == -2 ) closeProgressor();
	else progressor->setProgress( (int)progress );
}

void MyWidget::openProgressor( const QString& str ) {
	statusBar()->message( str );
	setLabelText( labelMode, str );
	progressor->reset();
	if( !progressor->isVisible() ) progressor->show();
}

void MyWidget::closeProgressor( void ) {
	statusBar()->clear();
	if( progressor->isVisible() ) progressor->hide();
}

void MyWidget::setMode( const QString& text, double sec ) {
	setLabelText( labelMode, text );
	Time t = sec;
	setLabelText( labelModeTime,t.toString() );
	if( isScenario ) {
		t = scenarioTime+sec;
		setLabelText( labelScenarioTime, t.toString() );
	} else setLabelText( labelScenarioTime, "" );
}

void MyWidget::setScenario( const QString& text, double sec ) {
	setLabelText( labelScenario, text );

	scenarioTime = sec;
//	Time t = sec;
//	setLabelText(labelScenarioTime, t.toString());
}

/*
void MyWidget::resizeEvent( QResizeEvent* e ) {

	int _chng = 0;
	if( _winWidth != e->size().width() ) {
		_winWidth = e->size().width();
		_chng = 1;
	}
	if( _winHeight != e->size().height() ) {
		_winHeight = e->size().height();
		_chng = 1;
	}
}
*/

/*
void MyWidget::moveEvent( QMoveEvent* e ) {
	_winX = e->pos().x();
	_winY = e->pos().y();
	jdebug( "MyWidget::MoveEvent: %d %d",_winX,_winY );

}
*/

void MyWidget::change( void ) {
	bool first = false;
	loadConst( first );
}

void MyWidget::fitToFont( void ) {
	labelApert->setFixedSize( labelApert->sizeHint() );
	labelDoor->setFixedSize( labelDoor->sizeHint() );
	labelIllum->setFixedSize( labelIllum->sizeHint() );
	labelHV->setFixedSize( labelHV->sizeHint() );
	labelTmpr->setFixedSize( labelTmpr->sizeHint() );
	labelZenit->setFixedSize( labelZenit->sizeHint() );
	labelStar->setFixedSize( labelStar->sizeHint() );
	digitalclock->fitToFont();

	labelMode->setFixedSize( labelMode->sizeHint() );
	labelScenario->setFixedSize( labelScenario->sizeHint() );
	labelModeTime->setFixedSize( labelModeTime->sizeHint() );
	labelScenarioTime->setFixedSize( labelScenarioTime->sizeHint() );
}

void MyWidget::loadConst( bool first = true ) {
	if( isLeafChanged( cfgFile,"Preferences",
			"Text window settings",
			"WindowGeometry" ) ) {
		const char* _geometry = leafDataToStr( cfgFile,
				"Preferences",
				"Text window settings",
				"WindowGeometry" )->data();
		if( !strcasecmp( _geometry,"AUTO" ) ) {
			_winX = 0;
			_winY = 0;
			_winWidth = qApp->desktop()->width()/2;
			_winHeight = qApp->desktop()->height()-50;
		} else {
			int key = sscanf( _geometry,"%d%d%d%d", &_winX, &_winY,
				&_winWidth, &_winHeight );
			if( key != 4 ) {
				QString warn = 
					"TextWindow: Invalid geometry! \nWill be modified.";
				showStrongMessage( warn );
				_winX = 0;
				_winY = 0;
				_winWidth = qApp->desktop()->width()/2;
				_winHeight = qApp->desktop()->height()-50;
			}
		}
		if( !first ) {
			setGeometry( _winX+10, _winY+68,
					_winWidth, _winHeight );
//   Pochemu +10 i +68 NE ZNAYU !!!!!
		}
	}

	if( isLeafChanged( cfgFile, "Preferences",
			"Text window settings",
			"FontSizeLarge" ) ) {
		_fontSizeLarge = leafDataToInt( cfgFile, "Preferences",
				"Text window settings",
				"FontSizeLarge" );
		qApp->setFont( QFont("Helvetica",_fontSizeLarge),true );
		if( existCommentDialog ) commentDialog->fitToFont();
		if( existCenteringDialog ) centeringDialog->fitToFont();
		if( existCenterCorrectionDialog ) trackCalibrationDialog->fitToFont();
		if( existTrackCalibrationDialog ) trackCalibrationDialog->fitToFont();
		if( !first ) fitToFont();
	}
	if( isLeafChanged( cfgFile, "Preferences",
			"Text window settings",
			"FontSizeSmall" ) ) {
		_fontSizeSmall = leafDataToInt( cfgFile, "Preferences",
				"Text window settings",
				"FontSizeSmall" );
		statusBar()->setFont( QFont("Helvetica",_fontSizeSmall),true );
		mainView->setFont( QFont("Helvetica",_fontSizeSmall),true );
		mainView->fitToFont();
	}

	QString* str = leafDataToStr( cfgFile, "Operations",
		"Star Pointing", "CoordinateSystem" );
	if( str->upper() == "RA" ) _showHA = 0;
	else _showHA = 1;
}

void MyWidget::slotDisableItems( void ) {
    	for( int i=0; i<n_dis; i++ ) {
		menuBar()->setItemEnabled( disabl_count[i], false );
	}
	if(!devmanager.isPlayBack()) menuBar()->setItemEnabled(pb_id,false);
	cnfg->setDisabled();
//	selectButton->setDisabled( true );
//	navigateButton->setDisabled( true );
	playButton->setDisabled( true );
	menuBar()->setItemEnabled( play_item,false );
	enableSelectScenario( false );
	menuBar()->setItemEnabled( break_item, true );
	if( existBackCorrectionDialog ) backCorrectionDialog->setButtonsEnabled( false );
	if( existCenterCorrectionDialog ) centerCorrectionDialog->setButtonsEnabled( false );
	if( existTrackCalibrationDialog ) trackCalibrationDialog->setButtonsEnabled( false );
	_updateHA = false;
}

void MyWidget::slotEnableItems( void ) {
	jdebug( "slot enable Items" );
	QApplication::beep();
	for( int i=0; i<n_dis; i++ ) {
		menuBar()->setItemEnabled( disabl_count[i], true );
	}
	if(!devmanager.isPlayBack()) menuBar()->setItemEnabled(pb_id,true);
	cnfg->setEnabled();
//	selectButton->setDisabled( false );
//	navigateButton->setDisabled( false );
	if( !isScenario ) {
		playButton->setDisabled( false );
		menuBar()->setItemEnabled( play_item,true );
		enableSelectScenario( true );
	}
	menuBar()->setItemEnabled( break_item, false );
	if( existBackCorrectionDialog ) backCorrectionDialog->setButtonsEnabled( true );
	if( existCenterCorrectionDialog ) centerCorrectionDialog->setButtonsEnabled( true );
	if( existTrackCalibrationDialog ) trackCalibrationDialog->setButtonsEnabled( true );
	_updateHA = true;
}

void MyWidget::disableDevice( bool disable ) {
    	for( int i=0; i<n_dev; i++ ) {
		menuBar()->setItemEnabled( device_enable[i], !disable );
	}
	playButton->setDisabled( disable );
	disableTracker( disable );
}

void MyWidget::disableTracker( bool disable ) {
	raPlus->setDisabled( disable );
	raMinus->setDisabled( disable );
	decPlus->setDisabled( disable );
	decMinus->setDisabled( disable );
}


MyWidget::~MyWidget() {
	jdebug( "................. MyWidget: destructor..." );
	killTimer( normalTimer );

	jdebug( "....delete toolBarDevice" );
	delete toolBarDevice;
//	jdebug( "....deletetoolBarDeviceHead " );
//	delete toolBarDeviceHead;
	jdebug( "....deletetoolBarObject " );
	delete toolBarObject;
	jdebug( "....delete toolBarProgram" );
	delete toolBarProgram;
	jdebug( "....delete toolBarButtons" );
	delete toolBarButtons;

	jdebug( "....delete mainView" );
	delete mainView;
	jdebug( "....delete progressor" );
	delete progressor;
/*
	jdebug( "....delete labelApert" );
	delete labelApert;
	jdebug( "....delete labelApertHead" );
	delete labelApertHead;
	jdebug( "....delete labelIllum" );
	delete labelIllum;
	jdebug( "....delete labelIllumHead" );
	delete labelIllumHead;
	jdebug( "....delete labelHV" );
	delete labelHV;
	jdebug( "....delete labelHVHead" );
	delete labelHVHead;
	jdebug( "....delete labelDoor" );
	delete labelDoor;
	jdebug( "....delete labelDoorHead" );
	delete labelDoorHead;
	jdebug( "....delete labelTmpr" );
	delete labelTmpr;
	jdebug( "....delete labelTmprHead" );
	delete labelTmprHead;

	jdebug( "....delete labelStar" );
	delete labelStar;
	jdebug( "....delete labelDiaphragm" );
	delete labelDiaphragm;
	jdebug( "....delete labelZenit" );
	delete labelZenit;
	jdebug( "....delete digitalclock" );
	delete digitalclock;

	jdebug( "....delete navigateButton" );
	delete navigateButton;
	jdebug( "....delete selectButton" );
	delete selectButton;

	jdebug( "....delete labelScenario" );
	delete labelScenario;
	jdebug( "....delete labelScenarioTime" );
	delete labelScenarioTime;
	jdebug( "....delete labelMode" );
	delete labelMode;
	jdebug( "....delete labelModeTime" );
	delete labelModeTime;
*/
	jdebug( "....delete playButtonGroup" );
	delete playButtonGroup;
/*
	jdebug( "....delete stopButton" );
	delete stopButton;
	jdebug( "....delete pauseButton" );
	delete pauseButton;
	jdebug( "....delete playButton" );
	delete playButton;
*/
	jdebug( "....delete file" );
	delete file;
	jdebug( "....delete measure" );
	delete measure;
	jdebug( "....delete tests" );
	delete tests;
	jdebug( "....delete help" );
	delete help;
	jdebug( "....delete config" );
	delete config;
	jdebug( "....delete tools" );
	delete tools;
	jdebug( "....delete functions" );
	delete functions;
	jdebug( "....delete aperture_set" );
	delete aperture_set;
	jdebug( "....delete scenario_set" );
	delete scenario_set;
	jdebug( "....delete cnfg" );
	delete cnfg;

	int i;
	jdebug( "....delete menu_help" );
	for( i=0; i<item_id; i++ ) {
		delete menu_help[i];
	}
	if( existTalkDialog ) {
		jdebug( "....delete talkDialog" );
		delete talkDialog;
		existTalkDialog = false;
	}
	if( existAdjustDialog ) {
		jdebug( "....delete adjustDialog" );
		delete adjustDialog;
		existAdjustDialog = false;
	}
	if( existSelectStarDialog ) {
		jdebug( "....delete selectStarDialog" );
		delete selectStarDialog;
		existSelectStarDialog = false;
	}
	if( existShowStarDialog ) {
		jdebug( "....delete showStarDialog" );
		delete showStarDialog;
		existShowStarDialog = false;
	}
	if( existCommentDialog ) {
		jdebug( "....delete CommentDialog" );
		delete commentDialog;
		existCommentDialog = false;
	}

	if( existCenteringDialog ) {
		jdebug( "....delete CenteringDialog" );
		delete centeringDialog;
		existCenteringDialog = false;
	}

	if( existTrackCalibrationDialog ) {
		jdebug( "....delete trackCalibrationDialog" );
		delete trackCalibrationDialog;
		existTrackCalibrationDialog = false;
	}

	if( existIllumDialog ) {
		jdebug( "....delete illumDialog" );
		delete illumDialog;
		existIllumDialog = false;
	}
	if( existLightDialog ) {
		jdebug( "....delete lightDialog" );
		delete lightDialog;
		existLightDialog = false;
	}
	if( existHVDialog ) {
		jdebug( "....delete hvDialog" );
		delete hvDialog;
		existHVDialog = false;
	}
	if( existBackCorrectionDialog ) {
		jdebug( "....delete backCorrectionDialog" );
		delete backCorrectionDialog;
		existBackCorrectionDialog = false;
	}
	if( existCenterCorrectionDialog ) {
		jdebug( "....delete centerCorrectionDialog" );
		delete centerCorrectionDialog;
		existCenterCorrectionDialog = false;
	}
	if( existHelpView ) {
		jdebug("...delete helpView" );
		delete helpView;
		existHelpView = false;
	}
}

void MyWidget::slotHighlighted( int id ) {
	if( id >= 0 ) {
		statusBar()->message( menu_help[id]->data(),4000 );
	}
}

void MyWidget::slotShowMessage( const char* _text, int ms ) {
	if( ms > 0 ) { statusBar()->message( _text,ms ); return; }
	if( ms < 0 ) { statusBar()->message( _text ); return; }
	if( ms == 0 ) { statusBar()->clear(); return; }
}

void MyWidget::slotShowMessage( const char* _text  ) {
	statusBar()->message( _text );
}

void MyWidget::slotHelp() {
	if( !existHelpView ) {
		QString fileName = helpFile;
		if( !testFileName( fileName,1 ) ) return;
		helpView = new HelpView( fileName,0,"Help" );
		CHECK_PTR(helpView);
		existHelpView = true;
	}
	helpView->exec();
}

void MyWidget::slotPlayBack() {
	if( !devmanager.isPlayBack() &&
			!devmanager.isCounting() ) { // start PlayBack
		QString pbDirName = *leafDataToStr( cfgFile, "General",
			"Inputs", "PlayBackDir" );
		QString fileExt = "*.";
		fileExt += pbInFileExt;
		QString fn = QFileDialog::getOpenFileName( dataPath+pbDirName,
							fileExt,this );
		if( testFileName( fn,0 ) ) {
			file->changeItem( pb_id, "End Play Back" );
			devmanager.playBack( fn );
		}
	} else if( !devmanager.isPlayBack() &&
			devmanager.isCounting() ) { // stop PlayBack
		devmanager.endPlayBack();
		file->changeItem( pb_id, "Play Back" );
	} else if( devmanager.isPlayBack() &&
			devmanager.isCounting() ) {
		devmanager.stopCounting();
		file->changeItem( pb_id, "Play Back" );
	}
}

void MyWidget::slotTalkDialog( void ) {
	if( !existTalkDialog ) {
		talkDialog = new TalkDialog();
		CHECK_PTR(talkDialog);
		existTalkDialog = true;
	}

	talkDialog->show();
}

void MyWidget::slotAdjustDialog( void ) {
	QString str;
	if( !existAdjustDialog ) {
		adjustDialog = new AdjustDialog();
		CHECK_PTR(adjustDialog);
		existAdjustDialog = true;
	}

	int _oldApert = devmanager.currentAperture();
	if( _oldApert == -1 ) {
		str = "Invalid current aperture";
		switch( showStrongWarning( str,"Ignore","Cancel" ) ) {
			case  QMessageBox::Abort  : return;
			case  QMessageBox::Ignore : break;
			default	: break;
		}
	}
	int _oldIllum = devmanager.isIllumOn();
	devmanager.turnIllumOn();
	devmanager.startAdjustAperture();
	adjustDialog->update();
	int ret = adjustDialog->exec();
	if( ret == QDialog::Accepted ) {
		devmanager.acceptAdjustAperture(1);
	}
	devmanager.setAperture( _oldApert,1,1 );
// !!! Nasil'stvenno stavim dazhe tu zhe aperturu, t.k. mogla byt'
// !!!sdvinuta na shagi!!!	
	devmanager.turnIllum( _oldIllum,1 );
}

void MyWidget::slotSelectStarDialog( void ) {
	QString str;
	if( !existSelectStarDialog ) {
		selectStarDialog = new SelectStarDialog();
		CHECK_PTR(selectStarDialog);
		existSelectStarDialog = true;
	} else selectStarDialog->update();

	selectButton->setDisabled( true );
	int ret = selectStarDialog->exec();
	if( ret == QDialog::Accepted ) {
		setLabelText( labelStar, star.getName() );
		setLabelText( labelZenit, star.getAirMass() );
	}
	selectButton->setDisabled( false );
}

void MyWidget::slotCommentDialog( void ) {
	if( !existCommentDialog ) {
		commentDialog = new CommentDialog();
		CHECK_PTR(commentDialog);
		existCommentDialog = true;
	}
	QString comment;
	commentDialog->update( &comment );

	int ret = commentDialog->exec();
	if( ret == QDialog::Accepted ) {
		devmanager.acceptComment( comment );
	}

}

void MyWidget::slotCenteringDialog( void ) {
	if( !existCenteringDialog ) {
		centeringDialog = new CenteringDialog();
		CHECK_PTR(centeringDialog);
		existCenteringDialog = true;
	}
	centeringDialog->update();

	centeringDialog->exec();
}

void MyWidget::slotShowStarDialog( void ) {
	int oldAperture = -1;
	if( withDevice ) oldAperture = devmanager.startShowStar();
	navigateButton->setDisabled( true );
	if( !existShowStarDialog ) {
		showStarDialog = new ShowStarDialog( _showHA );
		CHECK_PTR(showStarDialog);
		existShowStarDialog = true;
	} else showStarDialog->update( _showHA );

	showStarDialog->exec();
	if( withDevice ) devmanager.finishShowStar( oldAperture );
	navigateButton->setDisabled( false );
}

void MyWidget::timerEvent( QTimerEvent *e ) {
	if( isTimerEventLocked ) return;
	isTimerEventLocked = true;

	infoExportMutex.lock();
	if( ExportText::isChanged ) {
		ExportText::isChanged = false;
		mainView->showTextLine();
	}
	if( ExportTable::isChanged ) {
		ExportTable::isChanged = false;
		mainView->fillTable();
	}
	if( InfoExport::isProgressChanged ) {
		InfoExport::isProgressChanged = false;
		setProgress( InfoExport::progress.message,
				InfoExport::progress.progress );
	}

	if( InfoExport::isMessage ) {
		InfoExport::isMessage = false;
		slotShowMessage( InfoExport::message.data(),
				InfoExport::ms );
	}

//	if( isScenario != InfoExport::isScenario ) { // scenario state is changed
	if( InfoExport::isScenarioStateChanged ) { // scenario state is changed
		InfoExport::isScenarioStateChanged = false;
		isScenario = InfoExport::isScenario;
		if( !isScenario ) showStopScenario();
		else showPlayScenario();
	}

	if( InfoExport::isScenarioChanged ) {
		InfoExport::isScenarioChanged = false;
		setScenario( InfoExport::scenario.message,
				InfoExport::scenario.progress );
	}

	if( InfoExport::isModeChanged ) {
		InfoExport::isModeChanged = false;
		setMode( InfoExport::mode.message,
				InfoExport::mode.progress );
	}

	if( InfoExport::isStarPositionChanged ) {
		jdebug( "star position is changed" );
		jdebug( "oldPos = %d",oldStarPosition );
		jdebug( "newPos = %d",InfoExport::starPosition );
		InfoExport::isStarPositionChanged = false;
		if( oldStarPosition != InfoExport::starPosition ) {
			jdebug( "oldPos != newPos" );
			oldStarPosition = InfoExport::starPosition;
			displayStarPosition( oldStarPosition );
		}
	}

	infoExportMutex.unlock();

	if( StateExport::isStartCounting() ) {
		StateExport::clearStartCounting();
		slotDisableItems();
	}
	if( StateExport::isEndCounting() ) {
		StateExport::clearEndCounting();
		slotEnableItems();
		file->changeItem( pb_id, "Play Back" );
		slotShowMessage( "End of count" );
	}
	if( StateExport::isChanged() ) {
		StateExport::clearChanged();

		if( StateExport::isWithoutDevice() ) {
			StateExport::clearWithoutDevice();
			disableDevice( true );
		}
		if( StateExport::isWithDevice() ) {
			StateExport::clearWithDevice();
			disableDevice( false );
		}

		if( oldState.hv != StateExport::hv() ) {
			oldState.hv = StateExport::hv();
			slotTurnHV( oldState.hv );
		}
		if( oldState.safety != StateExport::safety() ) {
			oldState.safety = StateExport::safety();
			slotTurnSafety( oldState.safety );
		}
		if( oldState.illum != StateExport::illum() ) {
			oldState.illum = StateExport::illum();
			slotTurnIllum( oldState.illum );
		}
		if( oldState.light != StateExport::light() ) {
			oldState.light = StateExport::light();
			slotTurnLight( oldState.light );
		}
		if( oldState.door != StateExport::door() ) {
			oldState.door = StateExport::door();
			slotTurnDoor( oldState.door );
		}
		if( oldState.buttons != StateExport::buttons() ) {
			oldState.buttons = StateExport::buttons();
			slotTurnButtons( oldState.buttons );
		}
		if( oldState.aperture != StateExport::aperture() ) {
			oldState.aperture = StateExport::aperture();
			slotAperture( oldState.aperture );
		}
		if( oldState.temperature != StateExport::temperature() ) {
			oldState.temperature = StateExport::temperature();
			slotTemperature( oldState.temperature );
		}
	}
	digitalclock->showTime();
	if( _updateHA ) {
		star.updateCurrent();
		if( existShowStarDialog && _showHA ) showStarDialog->showHA();
	}
	setLabelText( labelZenit, star.getAirMass() );
	setLabelText( labelStar, star.getName() );
	isTimerEventLocked = false;
}

void MyWidget::slotIllumDialog( void ) {
	if( !existIllumDialog ) {
		illumDialog = new IllumDialog();
		CHECK_PTR(illumDialog);
		existIllumDialog = true;
	}
	illumDialog->update();
	illumDialog->show();
}

void MyWidget::slotLightDialog( void ) {
	if( !existLightDialog ) {
		lightDialog = new LightDialog();
		CHECK_PTR(lightDialog);
		existLightDialog = true;
	}
	lightDialog->update();
	lightDialog->show();
}

void MyWidget::slotHVDialog( void ) {
	if( !existHVDialog ) {
		hvDialog = new HVDialog();
		CHECK_PTR(hvDialog);
		existHVDialog = true;
	}
	hvDialog->update();
	hvDialog->show();
}

void MyWidget::slotBackCorrectionDialog( void ) {
	jdebug( "Slot BackCorrection Dialog!!!" );
	int ret = QMessageBox::information( 0,"background",
			"Point to the starless area of sky","Continue","Cancel" );
	if( ret != 0 ) {
		slotShowMessage( "Canceled...",4000 );
		return;
	}

	if( !existBackCorrectionDialog ) {
		backCorrectionDialog = new BackCorrectionDialog();
		CHECK_PTR(backCorrectionDialog);
		existBackCorrectionDialog = true;
	}
	backCorrectionDialog->update();

	int _oldApert = devmanager.currentAperture();
	if( _oldApert == -1 ) {
		QString str = "Invalid current aperture";
		switch( showStrongWarning( str,"Ignore","Cancel" ) ) {
			case  QMessageBox::Abort  : return;
			case  QMessageBox::Ignore : break;
			default	: break;
		}
	}
	ret = backCorrectionDialog->exec();
	if( ret == QDialog::Accepted ) {
		slotShowMessage( "Accepted...",4000 );
	} else {
		slotShowMessage( "Canceled...",4000 );
	}
	backCorrectionDialog->finish();
	devmanager.setAperture( _oldApert,1 );
}

void MyWidget::slotCenterCorrectionDialog( void ) {

	int ret = QMessageBox::information( 0,"centering",
			"Point to the star","Continue","Cancel" );
	if( ret != 0 ) {
		slotShowMessage( "Canceled...",4000 );
		return;
	}

	if( !existCenterCorrectionDialog ) {
		centerCorrectionDialog = new
			CorrectionDialog("Aperture Center","ShiftRA",
				"ShiftDec",CENTER );
		CHECK_PTR(centerCorrectionDialog);
		existCenterCorrectionDialog = true;
	}
	centerCorrectionDialog->update();

	int _oldApert = devmanager.currentAperture();
	if( _oldApert == -1 ) {
		QString str = "Invalid current aperture";
		switch( showStrongWarning( str,"Ignore","Cancel" ) ) {
			case  QMessageBox::Abort  : return;
			case  QMessageBox::Ignore : break;
			default	: break;
		}
	}
	ret = centerCorrectionDialog->exec();
	if( ret == QDialog::Accepted ) {
		slotShowMessage( "Accepted...",4000 );
	} else {
		slotShowMessage( "Canceled...",4000 );
	} 
	centerCorrectionDialog->finish();
	devmanager.setAperture( _oldApert,1 );
}

void MyWidget::slotTrackCalibrationDialog( void ) {
	int ret = QMessageBox::information( 0,"centering",
			"Point to the star","Continue","Cancel" );
	if( ret != 0 ) {
		slotShowMessage( "Canceled...",4000 );
		return;
	}

	if( !existTrackCalibrationDialog ) {
		trackCalibrationDialog = new
			CorrectionDialog("Track Calibration","SpeedRA",
				"SpeedDec",TRACK);
		CHECK_PTR(trackCalibrationDialog);
		existTrackCalibrationDialog = true;
	}
	trackCalibrationDialog->update();

	int _oldApert = devmanager.currentAperture();
	if( _oldApert == -1 ) {
		QString str = "Invalid current aperture";
		switch( showStrongWarning( str,"Ignore","Cancel" ) ) {
			case  QMessageBox::Abort  : return;
			case  QMessageBox::Ignore : break;
			default	: break;
		}
	}
	ret = trackCalibrationDialog->exec();
	if( ret == QDialog::Accepted ) {
		slotShowMessage( "Accepted...",4000 );
	} else {
		slotShowMessage( "Canceled...",4000 );
	} 
	trackCalibrationDialog->finish();
//	devmanager.endTrackCalibration();
	devmanager.setAperture( _oldApert,1 );
}

void MyWidget::displayStarPosition( int code ) {
	jdebug( "display Star position. Code=%d", code );
	switch( code ) {
		case 11 : labelDiaphragm->setPixmap( ld ); break;
		case  1 : labelDiaphragm->setPixmap( dd ); break;
		case 21 : labelDiaphragm->setPixmap( rd ); break;
		case 10 : labelDiaphragm->setPixmap( ll ); break;
		case  0 : labelDiaphragm->setPixmap( cc ); break;
		case 20 : labelDiaphragm->setPixmap( rr ); break;
		case 12 : labelDiaphragm->setPixmap( lu ); break;
		case  2 : labelDiaphragm->setPixmap( uu ); break;
		case 22 : labelDiaphragm->setPixmap( ru ); break;
		case -1 : labelDiaphragm->setPixmap( empt ); break;
	}
}

void MyWidget::keyPressEvent( QKeyEvent *k ) {
	if( devmanager.isTracking()
		&& (k->key() == Key_Control)
		&& !k->isAutoRepeat() ) { //ForceMove
		devmanager.forceTrack();
//		printf( "Force!\n" );
	}
	if( k->state() != ShiftButton ) return;
//	if( _disableKeyEvent ) return;
	if( !devmanager.withAutoGuide() ) return;
	if( devmanager.isTracking() ) return;
	if( k->isAutoRepeat() ) return;
	switch(k->key()) {

		case Key_Right	: //RA+
				if( moveRAplus() )
					raPlus->setDown(true);
				break;

		case Key_Left	: //RA-
				if( moveRAminus() )
					raMinus->setDown(true);
				break;

		case Key_Down	: //Dec-
				if( moveDecMinus() )
					decMinus->setDown(true);
				break;

		case Key_Up	: //Dec+
				if( moveDecPlus() )
					decPlus->setDown(true);
				break;
        }
}

void MyWidget::keyReleaseEvent( QKeyEvent *k ) {
//	debug( "ReleaseEvent!" );
	if( !devmanager.isTracking() 
		&& !StateExport::isUserTrack() ) return;
	if( k->isAutoRepeat() ) return;
//	debug( "ReleaseEvent after isTracking!" );
	switch(k->key()) {
		case Key_Right	: //RA+
		case Key_Left	: //RA-
		case Key_Down	: //Dec-
		case Key_Up	: //Dec+
				stopTrack();
				raPlus->setDown(false);
				raMinus->setDown(false);
				decPlus->setDown(false);
				decMinus->setDown(false);
				break;
	}
}

void MyWidget::closeEvent( QCloseEvent* e ) {
	jdebug( "CloseEvent!!!" );
	myQuit();	
	e->accept();
}
