/*--------------------------------------------------------------------------*/
/** @file tree.h
@brief  Class implementing the CFG tree organization: principles of its 
building, reading, searching info etc.

                  MASS project: TURBINA module file
                       header file for tree.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef TREE_H
#define TREE_H
#include <qobject.h>
#include <qstring.h>

class CFGTree;
class CFGBranch;
class CFGLeaf;

extern CFGLeaf leaf_empty;
extern CFGBranch branch_empty;

class Link {
private:
	Link* next;
	char id;	// 'l' or 'b' - leaf or branch
	void* data;	//  ,     CFGLeaf or CFGBranch

public:
	void initLink( char _id, void* _data, CFGBranch* _branch );	
	void initLink( char _id, void* _data, CFGTree* _tree );		

	virtual ~Link( void );
	const char* BranchName( void );
	const char* getLeafKey( void );
	char* getLeafData( void );
	char getId( void )		{ return id; }
	CFGBranch* getBranch( void ) {if(id=='b') return((CFGBranch*)data); return NULL; }

	CFGLeaf* getLeaf( void ) {if(id == 'l') return((CFGLeaf*)data); return NULL;}
	Link* getNext( void ) { return next; }
	void setNext( Link* _next ) { next = _next; }
};

class CFGLeaf {

private:
	const char *key;
	QString data;
	bool _changed;
	const char* validator_type;
	bool _enable_change;
	bool _for_root; // change enabled only for mass_root
	bool _count_disable; // disable or not v techenie nakopleniya
	QString help;

public:
	void initLeaf( const char*,
		bool for_root = true,
		bool enable_change = true,
		bool count_disable = true );
	void initLeaf( const char*,
		const char*,	// validator
		bool for_root = true,
		bool enable_change =true,
		bool count_disable = true );
	virtual ~CFGLeaf();

	bool isChanged( void ) { return _changed; }
	void setChanged( bool chng ) { _changed = chng; }
	bool isChangeEnabled( void ) { return _enable_change; }
	bool isForRoot( void ) { return _for_root; }
	bool isCountDisabled( void ) { return _count_disable; }
	const char* getKey( void ) { return key; }
	QString* getData( void ) { return &data; }
	QString* getHelp( void ) { return &help; }
	void setHelp( const char* );
	int replaceData( const QString& data_ );
	const char* getValidatorType() { return validator_type; }
};

class CFGBranch {

private:
	const char *name;
	Link *first_link; //    ,    link
	Link *last_link;
	Link *current_link;
	QString help;
	int n_leaf;		//         
	int n_subbranch; //     

public:
	void initBranch( const char* _name );

	virtual ~CFGBranch( void );
	void addLink( Link* );
	QString* getHelp( void ) { return &help; }
	void setHelp( const char* );
	CFGBranch* findBranch( const char* );
	CFGLeaf* findLeaf( const char* ); //     
	QString* findLeaf_str( const char* ); //       
	char* findLeaf_ch( const char* );
	int replaceLeaf( const char*,const char* );
	int replaceChanged( const char*, bool isChanged );
	int setLeafHelp( const char*,const char* );
	void toFirst( void ) { current_link = first_link; }
	void toNext( void ) { current_link = current_link->getNext(); }
	Link* getCurrentLink( void ) { return current_link; }
	const char* getName( void ) { return name; }
	int getNLeaf( void ) { return n_leaf; }
	int getNSubBranch( void ) { return n_subbranch; }
	int copyBranch( CFGBranch*,CFGBranch* );
};                        

class CFGTree : public QObject {
	Q_OBJECT
private:
	Link* first_link;
	Link* current_link;
	Link* last_link;
	int _changed;

public:
	CFGTree( void );
	virtual ~CFGTree( void );
	void addLink( Link* );
	CFGBranch* findBranch( const char* );
	void toFirst( void ) { current_link = first_link; } 
	void toNext( void ) { current_link = current_link->getNext(); }
	Link* getCurrentLink( void ) { return current_link; }
	const char* CurrentName( void ) { return current_link->getBranch()->getName(); }
	int copyTree( CFGTree*,CFGTree* );
	int readCfg ( const char* );
	int saveToFile( const char* );
	void emitSignalChanged( void );
	int isChanged( void ) {return( _changed ); }
	void setSaved( void ) {_changed = 0;}
	void setChanged( int chng  ) {_changed = chng;}
signals:
	void changed( void );
};


#endif

